
#ifndef AY38914_H
#define AY38914_H

#include <windows.h>
#include "AY38914_Registers.h"
#include "AY38914_Channel.h"
#include "AudioOutputLine.h"
#include "IODevice.h"
#include "MemoryAlias.h"
#include "Peripheral.h"
#include "Processor.h"
#include "PlugIn.h"

/**
 * The AY-3-8914 chip in the Intellivision, also know as the Programmable
 * Sound Generator (PSG).
 *
 * @author Kyle Davis
 */
class AY38914 : public Processor, public Peripheral
{

    friend class AY38914_Registers;

    public:
        AY38914();
        void init(AudioOutputLine* aol, INT32 location, IODevice* io0, IODevice* io1);
        INT32 getMemoryCount();
        void getMemory(INT32 i, Memory** m);
        INT32 getProcessorCount();
        void getProcessor(INT32 i, Processor** p);
        INT32 getClockSpeed();
        void reset();
        INT32 tick();
        void setClockDivisor(INT32 clockDivisor);
        INT32 getClockDivisor();
        void changePlugIn(PlugIn* plugIn);
        void setIODevice0(IODevice* io0);
        void setIODevice1(IODevice* io1);

/*
    public void save(SaveOutputStream os) throws IOException {
    public void load(LoadInputStream is) throws IOException {
*/

    private:
        AudioOutputLine*    aol;
        IODevice*           ioDevice0;
        IODevice*           ioDevice1;

        //divisor for slowing down the clock speed for the AY38914
        INT32 clockDivisor;

        //plug in device
        PlugIn*           plugIn;

        //registers and channels
        AY38914_Registers   registers;
        MemoryAlias         registerAlias;
        AY38914_Channel     channel0;
        AY38914_Channel     channel1;
        AY38914_Channel     channel2;

        //cached total output sample
        BOOL  cachedTotalOutputIsDirty;
        INT32 cachedTotalOutput;

        //envelope data
        BOOL  envelopeIdle;
        INT32 envelopePeriod;
        INT32 envelopePeriodValue;
        INT32 envelopeCounter;
        INT32 envelopeVolume;
        BOOL  envelopeHold;
        BOOL  envelopeAltr;
        BOOL  envelopeAtak;
        BOOL  envelopeCont;

        //noise data
        BOOL  noiseIdle;
        INT32 noisePeriod;
        INT32 noisePeriodValue;
        INT32 noiseCounter;

        //data for random number generator, used for white noise accuracy
        INT32 random;
        BOOL  noise;

        //output amplitudes for a single channel
        static const INT32 amplitudes16Bit[16];

};

#endif

