
#include "AudioMixer.h"
#include "AudioOutputLine.h"

AudioMixer::AudioMixer() {}

AudioMixer::AudioMixer(ProcessorBus* pb)
{
    init(pb);
}

void AudioMixer::init(ProcessorBus* pb)
{
    lineCount = 0;
    highQualityMixing = TRUE;
    processorBus = pb;
}

INT32 AudioMixer::getClockSpeed() {
    return aod->getSampleRate();
}

void AudioMixer::createLine(AudioOutputLine** line) {
    (*line) = new AudioOutputLine(this);
}

void AudioMixer::releaseLine(AudioOutputLine** line) {
    delete (*line);
}

void AudioMixer::reset() {
    for (INT32 i = 0; i < lineCount; i++)
        lines[i]->reset();
}

INT32 AudioMixer::tick() {
    INT32 totalSample = 0;

    //mix and flush the sample buffers
    for (INT32 i = 0; i < lineCount; i++) {
        AudioOutputLine* nextLine = lines[i];

        //calculate the sample for this line
        if (highQualityMixing) {
            nextLine->sampleBuffer += ((INT64)nextLine->currentSample) *
                    (INT64)(processorBus->clock - nextLine->lastWriteClockMark);
            totalSample += (INT32)(nextLine->sampleBuffer /
                    processorBus->clock);
        }
        else
            totalSample += nextLine->currentSample;

        //clear the sample buffer for this line
        nextLine->sampleBuffer = 0;
        nextLine->lastWriteClockMark = 0;
    }
    processorBus->clock = 0;

    totalSample = totalSample / lineCount;

    //apply the saturation clipping to correctly model the
    //cross-channel modulation that occurs on a real Intellivision
    if (totalSample > 0x6000)
        totalSample = 0x6000 + ((totalSample - 0x6000)/6);

    aod->playSample(totalSample);

    return 1;
}

void AudioMixer::changePlugIn(PlugIn* plugIn) {
    this->plugIn = plugIn;
    if (this->plugIn == NULL)
        this->aod = NULL;
    else
        this->aod = plugIn->getAudioOutputDevice();
}

