
#include <iostream>
#include "EmulationDirector.h"
#include "Intellivision.h"
#include "kailleraclient.h"

using namespace std;

EmulationDirector::EmulationDirector()
    : networkGame(FALSE),
      player1Remote(FALSE),
      player2Remote(FALSE)
{}

void EmulationDirector::init(Intellivision* inty)
{
    this->inty = inty;
}

void EmulationDirector::setNetworkGame(BOOL b)
{
    networkGame = b;
}

void EmulationDirector::setPlayerOneRemote(BOOL b)
{
    player1Remote = b;
}

void EmulationDirector::setPlayerTwoRemote(BOOL b)
{
    player2Remote = b;
}

INT32 EmulationDirector::getProcessorCount() {
    return 1;
}

void EmulationDirector::getProcessor(INT32 i, Processor** p) {
    (*p) = this;
}

INT32 EmulationDirector::getMemoryCount() {
    return 0;
}

void EmulationDirector::getMemory(INT32 i, Memory** m) {
    (*m) = NULL;
}

void EmulationDirector::reset() {
    for (INT32 i = 0; i < 6; i++)
        buttonPressed[i] = FALSE;
}

INT32 EmulationDirector::tick() {
    //poll the input device
    inputDevice->poll();

    //poll the hand controllers and ECS keyboard, if necessary
    inty->player1Controller->poll();
    inty->player2Controller->poll();
    if (inty->ecsInUse) {
        inty->ecs.keyboard.poll();
        if (networkGame) {
            byte values[72];
            ZeroMemory(values, sizeof(values));
            values[17] = 0xFF;

            //send the ECS keyboard state
            if (!player1Remote || !player2Remote) {
                values[0] = (byte)inty->ecs.keyboard.rowInputValues[0];
                values[1] = (byte)inty->ecs.keyboard.rowInputValues[1];
                values[2] = (byte)inty->ecs.keyboard.rowInputValues[2];
                values[3] = (byte)inty->ecs.keyboard.rowInputValues[3];
                values[4] = (byte)inty->ecs.keyboard.rowInputValues[4];
                values[5] = (byte)inty->ecs.keyboard.rowInputValues[5];
                values[6] = (byte)inty->ecs.keyboard.rowInputValues[6];
                values[7] = (byte)inty->ecs.keyboard.rowInputValues[7];
            }

            //send the hand controller states
            if (!player1Remote)
                values[8] = inty->player1Controller->inputValue;
            else if (!player2Remote)
                values[8] = inty->player2Controller->inputValue;

            if (kailleraModifyPlayValues(values, 9) == -1) {
                inty->turnOff();
                return 1;
            }

            //update the input states
            inty->player1Controller->inputValue = values[8];
            inty->player2Controller->inputValue = values[17];
            inty->ecs.keyboard.rowInputValues[0] = (byte)(values[0] |
                    values[9]);
            inty->ecs.keyboard.rowInputValues[1] = (byte)(values[1] |
                    values[10]);
            inty->ecs.keyboard.rowInputValues[2] = (byte)(values[2] |
                    values[11]);
            inty->ecs.keyboard.rowInputValues[3] = (byte)(values[3] |
                    values[12]);
            inty->ecs.keyboard.rowInputValues[4] = (byte)(values[4] |
                    values[13]);
            inty->ecs.keyboard.rowInputValues[5] = (byte)(values[5] |
                    values[14]);
            inty->ecs.keyboard.rowInputValues[6] = (byte)(values[6] |
                    values[15]);
            inty->ecs.keyboard.rowInputValues[7] = (byte)(values[7] |
                    values[16]);
        }
    }
    else if (networkGame) {
        byte values[8];
        values[1] = 0xFF;

        //send the hand controller states
        if (!player1Remote)
            values[0] = inty->player1Controller->inputValue;
        else if (!player2Remote)
            values[0] = inty->player2Controller->inputValue;

        if (kailleraModifyPlayValues(values, 1) == -1) {
            inty->turnOff();
            return 1;
        }

        //update the input states
        inty->player1Controller->inputValue = values[0];
        inty->player2Controller->inputValue = values[1];
    }

    //check the "turn off" button
    if (inputDevice->getSignalValue(inputSignals[0]) ==
            InputDevice::SIGNAL_ON)
        inty->turnOff();
    
    //check the "pause" button
    if (inputDevice->getSignalValue(inputSignals[1]) == InputDevice.SIGNAL_ON) {
        if (!buttonPressed[1]) {
            buttonPressed[1] = TRUE;
            BOOL paused = inty->processorBus.isPaused();
            inty->processorBus.setPaused(!paused);
            inty->stic.setMessage((!paused ? "PAUSED" : NULL));
        }
    }
    else
        buttonPressed[1] = FALSE;
    
/*
    if (inputDevice->getSignalValue(inputSignals[2]) == InputDevice.SIGNAL_ON) {
        if (!buttonPressed[2]) {
            buttonPressed[2] = TRUE;
            try {
                inty->saveGame();
                inty->stic.setMessage(savedMessage, 1);
            }
            catch (IOException ioe) {
                inty->stic.setMessage(saveFailedMessage, 1);
            }
        }
    }
    else
        buttonPressed[2] = FALSE;
    
    if (inputDevice->getSignalValue(inputSignals[3]) == InputDevice.SIGNAL_ON) {
        if (!buttonPressed[3]) {
            buttonPressed[3] = TRUE;
            try {
                inty->loadGame();
                inty->stic.setMessage(loadedMessage, 1);
            }
            catch (IOException ioe) {
                inty->stic.setMessage(loadFailedMessage, 1);
            }
        }
    }
    else
        buttonPressed[3] = FALSE;
*/
    
    //check the "reset" button
    if (inputDevice->getSignalValue(inputSignals[4]) == InputDevice.SIGNAL_ON)
        inty->reset();
    
/*
    if (inputDevice->getSignalValue(inputSignals[5]) == InputDevice.SIGNAL_ON) {
        if (!buttonPressed[5]) {
            buttonPressed[5] = TRUE;
            try {
                inty->writeSnapshot();
            }
            catch (IOException ioe) {
                inty->stic.setMessage(screenCaptureFailedMessage, 1);
            }
        }
    }
    else
        buttonPressed[5] = FALSE;
*/

    return 1;
}

INT32 EmulationDirector::getClockSpeed() {
    return 120;
}

void EmulationDirector::changePlugIn(PlugIn* plugIn) {
    this->plugIn = plugIn;
    if (this->plugIn == NULL)
        this->inputDevice = NULL;
    else {
        this->inputDevice = plugIn->getInputDevice();
        InputSignal* defaultSignal;
        for (INT32 i = 0; i < 6; i++) {
            inputDevice->getDefaultConfiguration(InputDevice::ID_DIRECTOR, i,
                    &defaultSignal);
            this->inputSignals[i] = defaultSignal;
        }
    }
}

