
#include <string.h>
#include <stdio.h>
#include "ROM.h"

ROM::ROM()
    : image(NULL)
{}

ROM::ROM(INT32 size, INT32 location)
{
    init(size, location);
}

void ROM::init(INT32 size, INT32 location)
{
    this->image = new INT32[size];
    this->size = size;
    this->location = location;
}

void ROM::loadImage(INT32* image)
{
    memcpy(this->image, image, size*sizeof(INT32));
}

ROM::~ROM()
{
    if (image != NULL)
        delete[] image;
}

void ROM::reset()
{
    //ROM can't be changed, so there's nothing to reset
}

INT32 ROM::getLocation() {
    return location;
}

INT32 ROM::peek(INT32 location) {
    return image[location-this->location];
}

void ROM::poke(INT32 location, INT32 value)
{
    //can't change ROM
}

INT32 ROM::getSize()
{
    return size;
}

void ROM::loadROMImage(INT32** romImage, INT32* size, const CHAR* fileName,
        BOOL is16bit)
{
    INT32 currentSize = 4096;
    INT32* image = new INT32[currentSize];
    INT32 totalSize = 0;
    INT32 nextRead;

    FILE* file = fopen(fileName, "rb");
    nextRead = fgetc(file);
    while (nextRead != EOF) {
        if (totalSize >= currentSize) {
            INT32* newImage = new INT32[currentSize<<1];
            memcpy(newImage, image, currentSize*sizeof(INT32));
            currentSize <<= 1;
            delete[] image;
            image = newImage;
        }

        if (is16bit)
            image[totalSize] = (nextRead << 8) | fgetc(file);
        else
            image[totalSize] = nextRead;

        totalSize++;
        nextRead = fgetc(file);
    }

    INT32* finalImage = new INT32[totalSize];
    memcpy(finalImage, image, totalSize*sizeof(INT32));
    delete[] image;
    (*romImage) = finalImage;
    (*size) = totalSize;
}

void ROM::loadROM(ROM** rom, INT32 location, const CHAR* fileName,
        BOOL is16bit)
{
    INT32* romImage;
    INT32 size;
    loadROMImage(&romImage, &size, fileName, is16bit);
    (*rom) = new ROM(size, location);
    (*rom)->loadImage(romImage);
    delete[] romImage;
}

