
#ifndef ECS_H
#define ECS_H

#include <windows.h>
#include "AY38914.h"
#include "BankSwitchedROM.h"
#include "Peripheral.h"
#include "AudioOutputLine.h"
#include "ECSKeyboard.h"
#include "RAM.h"

class Intellivision;
class EmulationDirector;

class ECS : public Peripheral
{

    friend class Intellivision;
    friend class EmulationDirector;

    public:
        ECS();
        ECS(AudioOutputLine* aol);
        ~ECS();
        void init(AudioOutputLine* aol);
        INT32 getProcessorCount();
        void getProcessor(INT32 i, Processor** p);
        INT32 getMemoryCount();
        void getMemory(INT32 i, Memory** m);

    private:
        void init();
        void setECSROMImage(INT32* ecsImage);
        BOOL hasECSROMImage();
        void setPSGClockDivisor(INT32 divisor);

        AudioOutputLine*    aol;
        ECSKeyboard         keyboard;
        AY38914             psg2;
        BOOL                hasECSROM;
        BankSwitchedROM     bank0;
        BankSwitchedROM     bank1;
        BankSwitchedROM     bank2;
        RAM                 ramBank;
        RAM                 uart;
        Memory**            memorySegments;

};

#endif
