
#include <iostream>
#include "Xanadu.h"
#include "Intellivision.h"
#include "CartridgeType.h"
#include "CartridgeTypeFactory.h"
#include "CRC32.h"
#include "ROM.h"
#include "Win32PlugIn.h"
#include "XanaduMainWindow.h"

using namespace std;

const UINT32 Xanadu::CRC_EXECUTIVE_ROM = 0xCBCE86F7;
const UINT32 Xanadu::CRC_GROM          = 0x683A4158;
const UINT32 Xanadu::CRC_ECS_ROM       = 0xEA790A06;
const UINT32 Xanadu::CRC_IVOICE_ROM    = 0x0DE7579D;
const CHAR* Xanadu::FILENAME_EXECUTIVE_ROM = "exec.bin";
const CHAR* Xanadu::FILENAME_GROM          = "grom.bin";
const CHAR* Xanadu::FILENAME_ECS_ROM       = "ecs.bin";
const CHAR* Xanadu::FILENAME_IVOICE_ROM    = "ivoice.bin";

INT32 main(INT32 argc, CHAR* argv[])
{
/*
    if (argc != 2) {
        cout << "Usage:  Xanadu <cartridge_file>\n\n";
        return -1;
    }
*/

    Xanadu xanadu;
    return xanadu.run(argv[1]);
}

Xanadu::Xanadu()
/*
    : cart(NULL),
      execImage(NULL),
      gromImage(NULL),
      ecsImage(NULL),
      ivoiceImage(NULL)
*/
{
}

Xanadu::~Xanadu() {
/*
    //clean up
    if (cart != NULL)
        delete cart;
    if (execImage != NULL)
        delete[] execImage;
    if (gromImage != NULL)
        delete[] gromImage;
    if (ecsImage != NULL)
        delete[] ecsImage;
    if (ivoiceImage != NULL)
        delete[] ivoiceImage;
*/
}

INT32 Xanadu::run(const CHAR* filename) {

    XanaduMainWindow win;
    win.setVisible(TRUE);
/*
    cout << "\n";

    //TODO: verify that the executive rom file exists
    //verify the CRC of the executive rom file
    UINT32 crc = CRC32::getCrc(FILENAME_EXECUTIVE_ROM);
    if (crc != CRC_EXECUTIVE_ROM) {
        cout << "ERROR:  File 'exec.bin' does not contain the correct " <<
                "Executive ROM\n\n";
        return -1;
    }

    //load the executive rom file
    INT32 size;
    ROM::loadROMImage(&execImage, &size, FILENAME_EXECUTIVE_ROM, TRUE);

    //TODO: verify that the grom rom file exists
    //verify the CRC of the grom rom file
    crc = CRC32::getCrc(FILENAME_GROM);
    if (crc != CRC_GROM) {
        cout << "ERROR:  File 'grom.bin' does not contain the correct GROM\n\n";
        delete[] execImage;
        return -1;
    }

    //load the grom rom file
    ROM::loadROMImage(&gromImage, &size, FILENAME_GROM, FALSE);

    //TODO: verify that the specified cartridge file exists
    //verify that the specified cartridge file contains a known cartridge
    crc = CRC32::getCrc(filename);
    CartridgeType* cartType = CartridgeTypeFactory::FACTORY.getCartridgeType(
            crc);
    if (cartType == NULL) {
        cout << "ERROR:  File '" << filename << "' does not contain a " <<
                "known cartridge\n\n";
        delete[] execImage;
        delete[] gromImage;
        return -1;
    }

    //load the cartridge file
    cart = Cartridge::loadCartridge(filename);

    //if the cartridge requires the ECS
    if (cart->requiresECS()) {
        //TODO: verify that the ECS rom file exists
        //verify the CRC of the ECS rom file
        crc = CRC32::getCrc(FILENAME_ECS_ROM);
        if (crc != CRC_ECS_ROM) {
            cout << "ERROR:  Cartridge '" << cartType->getName() <<
                    "' found in file '" << filename <<
                    "' requires ECS support, but the file 'ecs.bin' does " <<
                    "not contain the correct ECS ROM\n\n";
            return -1;
        }

        //load the ECS rom file
        ROM::loadROMImage(&ecsImage, &size, FILENAME_ECS_ROM, FALSE);
    }

    //if the cartridge can use the Intellivoice
    if (cart->usesIntellivoice()) {
        //TODO: check to see if the intellivoice rom file exists
        //TODO: verify the CRC of the Intellivoice rom file
        crc = CRC32::getCrc(FILENAME_ECS_ROM);
        if (crc != CRC_ECS_ROM) {
            cout << "WARNING:  File 'ivoice.bin' does not contain the " <<
                    "correct Intellivoice ROM.\n";
            cout << "Intellivoice support disabled\n\n";
        }
        else
            //load the intellivoice ROM file
            ROM::loadROMImage(&ivoiceImage, &size, FILENAME_IVOICE_ROM, FALSE);
    }

    //create the emulator
    Intellivision inty;
    inty.setExecImage(execImage);
    inty.setGROMImage(gromImage);
    //if (ecsImage != NULL)
        //inty.setECSROMImage(ecsImage);
    //if (ivoiceImage != NULL)
        //inty.setIVoiceROMImage(ivoiceImage);

    //init the plug-in
    Win32PlugIn plugIn;
    inty.setPlugIn(&plugIn);
    inty.insertCartridge(cart);

    cout << "Playing Cartridge:  " << cart->getName() << "\n\n";

    //turn on the Intellivision
    inty.turnOn();
/*
    //TODO: verify that it turned on OK
    INT32 result = inty.turnOn();
    if (result) {
        cout << "ERROR:  " << Win32PlugIn::getErrorMessage(result) << "\n";
        return -1;
    }
*

    //run the emulation
    inty.run();

    //turn off the Intellivision
    inty.turnOff();
*/

    return 0;
}

